package net.pricefx.common.apibuilder.chart;

import net.pricefx.common.api.chart.ChartType;
import net.pricefx.common.apibuilder.AbstractBuilder;
import net.pricefx.common.apibuilder.IdGenerator;
import net.pricefx.common.apibuilder.Invocations;
import net.pricefx.common.apibuilder.MethodInvocation;

public abstract class AbstractChartBuilder <C extends AbstractChartBuilder<C>> extends AbstractBuilder {
	
	private Dictionary<C> dictionary;
	
	protected AbstractChartBuilder(final String classId, final IdGenerator idGenerator, final Invocations<MethodInvocation> invocations) {
		super(idGenerator.getNext(), classId, idGenerator, invocations);
	}
	
	@Override
	protected String addInvocation(final String methodName, final Object...arguments) {
		return super.addInvocation(methodName, arguments);
	}
	
	IdGenerator getIdGenerator() {
		return idGenerator;
	}
	
	Invocations<MethodInvocation> getInvocations() {
		return invocations;
	}
	
	@SuppressWarnings("unchecked")
	protected C getThis() {
		return (C) this;
	}
	
	public final Dictionary<C> getDictionary() {
		final String id = addInvocation("getDictionary");
		if (dictionary == null) {
			dictionary = new Dictionary<>(getThis(), id, "Dictionary");
		}
		return dictionary;
	}
	
	protected String invokeGetOptions() {
		return addInvocation("getOptions");
	}
	
	protected String invokeAddSeries() {
		return addInvocation("addSeries");
	}
	
	protected String invokeGetSeries() {
		return addInvocation("getSeries");
	}
	
	protected AuxLine<C> addAuxLine() {
		final String id = addInvocation("addAuxLine");
		return new AuxLine<>(getThis(), id, "AuxLine");
	}
	
	protected ChartBuilderResult build(final ChartType type) {
		addInvocation("build");
		
		final ChartBuilderResult r = new ChartBuilderResult();
		r.invocations = invocations.get();
		r.additionalInfo = type;
		return r;
	}
	
}
