package net.pricefx.common.apibuilder.chart;

import net.pricefx.common.api.chart.AuxLineColor;
import net.pricefx.common.apibuilder.BacklinkBuilder;
import net.pricefx.common.apibuilder.ObjectIdentity;

public class AuxLine<C extends ObjectIdentity> extends BacklinkBuilder<C> {

	public AuxLine(final C backLink, final String instanceId, final String classId) {
		super(instanceId, classId, backLink);
	}

	public AuxLine<C> setLabel(final String label) {
		addInvocation("setLabel",label);
		return this;
	}

	public AuxLine<C> setShowInLegend(final Boolean showInLegend) {
		addInvocation("setShowInLegend",showInLegend);
		return this;
	}
	
	public AuxLine<C> setHideDataLabels(final Boolean hideDataLabels) {
		addInvocation("setHideDataLabels",hideDataLabels);
		return this;
	}

	public Line<AuxLine<C>> withLine() {
		final String id = addInvocation("withLine");
		return new Line<>(this, id, "Line");
	}

	public SteppedLine<AuxLine<C>> withSteppedLine() {
		final String id = addInvocation("withSteppedLine");
		return new SteppedLine<>(this, id, "SteppedLine");
	}

	public VerticalLine<AuxLine<C>> withVerticalLine() {
		final String id = addInvocation("withVerticalLine");
		return new VerticalLine<>(this, id, "VerticalLine");
	}

	public AuxLine<C> setColor(final AuxLineColor color) {
		addInvocation("setColor",color);
		return this;
	}

	public static class SteppedLine<C extends ObjectIdentity> extends BacklinkBuilder<C> {

		public SteppedLine(final C backLink, final String instanceId, final String classId) {
			super(instanceId, classId, backLink);
		}

		public SteppedLine<C> addStep(final String x, final String y) {
			addInvocation("addStep",x, y);
			return this;
		}
	}
	
	public static class VerticalLine<C extends ObjectIdentity> extends BacklinkBuilder<C> {

		public VerticalLine(final C backLink, final String instanceId, final String classId) {
			super(instanceId, classId, backLink);
		}

		public VerticalLine<C> setXValue(final String xValue) {

			addInvocation("setXValue",xValue);
			return this;
		}
	}

	public static class Line<C extends ObjectIdentity> extends BacklinkBuilder<C> {

		public Line(final C backLink, final String instanceId, final String classId) {
			super(instanceId, classId, backLink);
		}

		public Line<C> setYIntercept(final String yIntercept) {
			addInvocation("setYIntercept",yIntercept);
			return this;
		}

		public Line<C> setSteepness(final String steepness) {
			addInvocation("setSteepness",steepness);
			return this;
		}
	}
	
}