package net.pricefx.logic.core.model;

import com.fasterxml.jackson.databind.node.ObjectNode;
import net.pricefx.common.api.IFormulaNature;
import net.pricefx.common.api.WorkflowFormulaNature;
import net.pricefx.common.api.WorkflowType;
import net.pricefx.pckg.transform.TransformWorkflowFormula;
import net.pricefx.pckg.transform.descriptor.TypeDescriptor;

import java.util.Map;

import static net.pricefx.common.api.PriceFxInterface.FIELD_UNIQUENAME;
import static net.pricefx.common.api.PriceFxInterface.FIELD_WFE_WORKFLOWTYPE;

public class WorkflowFormula extends AbstractFormula {

    public WorkflowFormula() {
    }

    public WorkflowFormula(ObjectNode logic) {
        super(logic);
    }

    @Override
    public AbstractFormulaElement createElement() {
        return new WorkflowFormulaElement();
    }

    @Override
    public AbstractFormulaElement createElement(Map node) {
        return new WorkflowFormulaElement(node);
    }


    @Override
    public final boolean isPricingLogic() {
        return false;
    }

    @Override
    public final boolean isWorkflowLogic() {
        return true;
    }

    @Override
    public IFormulaNature getFormulaNatureObject() {
        return WorkflowFormulaNature.getViaDBValue(getFormulaNature());
    }

    public final String getWorkflowTypeStr() {
        String workflowTypeStr = getTextValue(FIELD_WFE_WORKFLOWTYPE);

        // TODO this is due to backwards compatibility. To be removed one day...
        if (workflowTypeStr == null) {
            workflowTypeStr = getTextValue(FIELD_UNIQUENAME);
        }

        return workflowTypeStr;
    }

    public final WorkflowType getWorkflowType() {
        return WorkflowType.getViaDBValue(getWorkflowTypeStr());
    }

    public final void setWorkflowType(String workflowTypeStr) {
        setTextValue(FIELD_WFE_WORKFLOWTYPE, workflowTypeStr);
    }

    @Override
    public TypeDescriptor getDescriptor() {
        return TransformWorkflowFormula.DESCRIPTOR;
    }
}

